""" Modulo con las funciones para fitear impedancias"""
import iamend_ci.theo as theo


import numpy as np
from scipy import optimize
import tkinter as tk
from tkinter import filedialog
root = tk.Tk()
root.withdraw()
import plotly.graph_objs as go
import plotly
plotly.offline.init_notebook_mode(connected=True)
from plotly.subplots import make_subplots
import os
import csv





def semilogfit(x,Y,titulo):
    """ fiteo y plot (plotly) varias muestras """
    
    fig =make_subplots(rows=1, cols=1 ,print_grid=False)
    for b,a in enumerate(Y):
        if b==0:
            trace0 = go.Scatter(
            x = x,
            y = a,
            name='exp',
            mode = 'markers')
            fig.append_trace(trace0, 1, 1)

        else:
            trace0 = go.Scatter(
            x = x,
            y = a,
            mode='lines',
            name=titulo)
            fig.append_trace(trace0, 1, 1)
    fig["layout"]["xaxis"].update( type='log',zeroline=False , autorange=True   )
    fig["layout"]["yaxis"].update( type='linear',zeroline=False , autorange=True   )
    return(fig)






def z1(f,bo,datacorr,n,dpatron,sigma,mur):
    """z1 (frecuencia, bobina, datacorr, n, dpatron,sigma, mur)
    Ajuste del lift-off
    Parameters
    ----------
    f : array_like, vector con las frecuencias
    datacorr : array_like, matriz con las mediciones
    bo: array_like, vector con los parametros geometricos de la bobina
    datacorr: array_like, matrix con las mediciones corregidas y nromalizadas
    n : int, indice de la medicion 
    dpatron: float, espesor muestra
    sigma : float, conductividad muestra
    mur: float, permeabilidad muestra
    """    
    def funz1(x,b):
        r1=bo[0]
        r2=bo[1]
        dh=bo[2]
        N=bo[3]
        z1=b
        bob=[r1,r2,dh,N,z1,1]
        return theo.dzD(x,bob,sigma,dpatron,mur,3000).imag/x0
    #[f,z0,dzucorr,w]
    l0=bo[-1]
    dzucorrnorm=datacorr[0][n]
    w=2*np.pi*f
    x0=w*l0
      
    xmeas=f
    ymeas=dzucorrnorm.imag   
    fpar, fcov=optimize.curve_fit(funz1, xmeas, ymeas, p0=[1.1e-3], bounds=(0,2e-3))
    

    z1eff=fpar[0]
    r1=bo[0]
    r2=bo[1]
    dh=bo[2]
    N=bo[3]
    boeff=[r1,r2,dh,N,z1eff,l0]
    yteo=theo.dzD(f,boeff,sigma,dpatron,mur,1500)
    yteo=yteo.imag/x0
    
    fig=semilogfit(f,[ymeas, yteo],'curve_fit{z1} '+datacorr[1][n])
    print('z1 =',fpar[0]*1000,'mm')
    return(fpar,fig)






def mu(f,bo,datacorr,n,sigma, z1eff, dpatron=1):
    """mu (frecuencia, bobina, datacorr, n, dpatron,sigma, mur)
    Ajuste de la permeabilidad

    Parameters
    ----------
    f : array_like, vector con las frecuencias
    datacorr : array_like, matriz con las mediciones
    bo: array_like, vector con los parametros geometricos de la bobina
    datacorr: array_like, matrix con las mediciones corregidas y nromalizadas
    n : int, indice de la medicion 
    dpatron: float, espesor muestra
    sigma : float, conductividad muestra
    z1eff: float, lift-off efectivo
    """    
    def funmu(x,a):
        bob=bo[:]
        bob[4]=z1eff
        return theo.dzD(x,bob,sigma,dpatron,a,1500).imag/x0
    
    #[f,z0,dzucorr,w]
    l0=bo[-1]
    dzucorrnorm=datacorr[0][n]
    w=2*np.pi*f
    x0=w*l0
      
    xmeas=f
    ymeas=dzucorrnorm.imag   
    fpar, fcov=optimize.curve_fit(funmu, xmeas, ymeas, p0=[1], bounds=(0,150))
    

    mur=fpar[0]
    yteo=theo.dzD(f,bo,sigma,dpatron,mur,1500)
    yteo=yteo.imag/x0
    
    fig=semilogfit(f,[ymeas, yteo],'curve_fit{mu} '+datacorr[1][n])
    print('mu =',fpar[0])
    return(fpar,fig)


#####################


#
#
#def mu(f,bo,dzcorrnorm,sigma,z1eff,dpatron=1,silent=1, lims=(1, 100),name='noname'):
#    def funmu(x,a):
#        bob=bo[:]
#        bob[4]=z1eff
#        return theo.dzD(x,bob,sigma,dpatron,a,1500).imag/x0
#    
#    l0=bo[-1]
#    w=2*np.pi*f
#    x0=w*l0
#    
#    xmeas=f
#    ymeas=dzcorrnorm.imag
#    fpar, fcov=optimize.curve_fit(funmu, xmeas, ymeas, p0=5, bounds=lims)
#    yteo=funmu(f,fpar[0])
#    res = ymeas-yteo
#    ss_res=np.sum(res**2)
#    ss_tot = np.sum((ymeas-np.mean(ymeas))**2)
#    rsqr=1-(ss_res/ss_tot)
#    if silent==0:    
#        print('mu =',fpar, '  perr =',perr)
#            
#    return([bo,f,dzcorrnorm,sigma,dpatron,fpar[0],rsqr,yteo,name])

#
#
#
#def semilog():
#    """ plot plotly semilog """
#    fig = go.Figure()
#    fig["layout"]["xaxis"].update( type='log',zeroline=False , autorange=True   )
#    fig["layout"]["yaxis"].update( type='linear',zeroline=False , autorange=True   )
#    return(fig)
#
#def z1r2(bo,aire,muestra,sep,dpatron,sigma,mur):
#    """ fiteo z1 r2 """
#    def funz1r2(x,a,b):
#        r1=bo[0]
#        r2=a
#        dh=bo[2]
#        N=bo[3]
#        z1=b
#        bob=[r1,r2,dh,N,z1,1]
#        return dztheo.dzHF(x,bob,sigma,mur,3000).imag/x0
#    dzucorr=solartron.dzcorr(aire,muestra,sep)
#    za=solartron.mean(solartron.read(aire,sep))    
#    z0=dzucorr[1]
#    x0=dzucorr[1].imag
#    r0=dzucorr[1].real
#    w=za[3]
#    f=za[0]
#    l0=dzucorr[1].imag/w
#    xmeas=f
#    ymeas=dzucorr[2].imag/x0    
#    fpar, fcov=optimize.curve_fit(funz1r2, xmeas, ymeas, p0=[bo[1],bo[4]], bounds=(0,5e-3))
#    r2eff=fpar[0]
#    z1eff=fpar[1]
#    r1=bo[0]
#    dh=bo[2]
#    N=bo[3]
#    boeff=[r1,r2eff,dh,N,z1eff,l0]
#    yteo=dztheo.dzD(f,boeff,sigma,dpatron,mur,3000)
#    yteo=yteo.imag/x0
#    semilogfit(f,[ymeas, yteo],'im(dzucorr vs dzteo) curve_fit{z1,r2}')
#    print('r2 =',fpar[0]*1000,'  z1 =',fpar[1]*1000)
#    return(fpar)
#
#def sigma(bo,aire,muestra,sep,dpatron,z1,mur):
#    """ fiteo sigma """
#    def funz1r2(x,b):
#        r1=bo[0]
#        r2=bo[1]
#        dh=bo[2]
#        N=bo[3]
#        bob=[r1,r2,dh,N,z1,1]
#        return dztheo.dzHF(x,bob,b,mur,3000).imag/x0    
#    dzucorr=solartron.dzcorr(aire,muestra,sep)
#    za=solartron.mean(solartron.read(aire,sep))
#    z0=dzucorr[1]
#    x0=dzucorr[1].imag
#    r0=dzucorr[1].real
#    w=za[3]
#    f=za[0]
#    l0=dzucorr[1].imag/w
#    xmeas=f
#    ymeas=dzucorr[2].imag/x0
#    fpar, fcov=optimize.curve_fit(funz1r2, xmeas, ymeas, p0=[1e6], bounds=((1e6,1e8)))
#    r1=bo[0]
#    r2=bo[1]
#    dh=bo[2]
#    N=bo[3]
#    boeff=[r1,r2,dh,N,z1,l0]
#    sigmaeff=fpar[0]
#    yteo=dztheo.dzD(f,boeff,sigmaeff,dpatron,mur,3000)
#    yteo=yteo.imag/x0
#    semilogfit(f,[ymeas, yteo],'im(dzucorr vs dzteo) curve_fit{z1}')
#    print( 'sigma =',fpar[0])
#    return(fpar)
#
#def sigmaz1(bo,aire,muestra,sep,dpatron,mur):
#    """ fiteo sigma z1 """
#    def funz1r2(x,a,b):
#        r1=bo[0]
#        r2=bo[1]
#        dh=bo[2]
#        N=bo[3]
#        z1=a
#        bob=[r1,r2,dh,N,z1,1]
#        return dztheo.dzHF(x,bob,b,mur,3000).imag/x0
#    
#    dzucorr=solartron.dzcorr(aire,muestra,sep)
#    za=solartron.mean(solartron.read(aire,sep))
#    z0=dzucorr[1]
#    x0=dzucorr[1].imag
#    r0=dzucorr[1].real
#    w=za[3]
#    f=za[0]
#    l0=dzucorr[1].imag/w
#    xmeas=f
#    ymeas=dzucorr[2].imag/x0
#    r1=bo[0]
#    r2=bo[1]
#    dh=bo[2]
#    N=bo[3]
#    fpar, fcov=optimize.curve_fit(funz1r2, xmeas, ymeas, p0=[bo[4],1e6], bounds=((0,5e-3),(1e6,1e8)))
#    fpar
#    r2eff=r2
#    z1eff=fpar[0]
#    boeff=[r1,r2eff,dh,N,z1eff,l0]
#    sigmaeff=fpar[1]
#    yteo=dztheo.dzD(f,boeff,sigmaeff,dpatron,mur,3000)
#    yteo=yteo.imag/x0
#    semilogfit(f,[ymeas, yteo],'im(dzucorr vs dzteo) curve_fit{z1,sigma}')
#    print('z1 =',fpar[0]*1000,'  sigma =',fpar[1]/1e6)
#    return(fpar)
#
#def muz1(bo,aire,muestra,sep,dpatron,sigma):
#    """ fiteo mu z1 """
#    def funmuz1(x,a,b):
#        r1=bo[0]
#        r2=bo[1]
#        dh=bo[2]
#        N=bo[3]
#        z1=a
#        bob=[r1,r2,dh,N,z1,1]
#        return dztheo.dzHF(x,bob,sigma,b,3000).imag/x0
#    
#    dzucorr=solartron.dzcorr(aire,muestra,sep)
#    za=solartron.mean(solartron.read(aire,sep))
#    z0=dzucorr[1]
#    x0=dzucorr[1].imag
#    r0=dzucorr[1].real
#    w=za[3]
#    f=za[0]
#    l0=dzucorr[1].imag/w
#    xmeas=f
#    ymeas=dzucorr[2].imag/x0
#    fpar, fcov=optimize.curve_fit(funmuz1, xmeas, ymeas, p0=[bo[4],1], bounds=((0,3e-3),(1,150)))
#    z1eff=fpar[0]
#    r1=bo[0]
#    r2=bo[1]
#    dh=bo[2]
#    N=bo[3]
#    boeff=[r1,r2,dh,N,z1eff,l0]
#    mueff=fpar[1]
#    yteo=dztheo.dzD(f,boeff,sigma,dpatron,mueff,3000)
#    yteo=yteo.imag/x0
#    #semilogfit(f,[ymeas, yteo],'im(dzucorr vs dzteo) curve_fit{z1,mu}')
#    print('z1 =',fpar[0]*1000,'  mu =',fpar[1])
#    return(fpar,f,ymeas,yteo)
#
#def mu2(bo,aire,muestra,sep,dpatron,sigma):
#    """ fiteo mu 2"""
#    def funmuz1(x,a):
#        return dztheo.dzHF(x,bo,sigma,a,3000).imag/x0    
#    dzucorr=solartron.dzcorr(aire,muestra,sep)
#    za=solartron.mean(solartron.read(aire,sep))
#    z0=dzucorr[1]
#    x0=dzucorr[1].imag
#    r0=dzucorr[1].real
#    w=za[3]
#    f=za[0]
#    l0=dzucorr[1].imag/w
#    xmeas=f
#    ymeas=dzucorr[2].imag/x0
#    fpar, fcov=optimize.curve_fit(funmuz1, xmeas, ymeas, p0=[1], bounds=(1,150))
#    mueff=fpar[0]
#    yteo=dztheo.dzD(f,bo,sigma,dpatron,mueff,3000)
#    yteo=yteo.imag/x0
#    semilogfit(f,[ymeas, yteo],'im(dzucorr vs dzteo) curve_fit{z1,mu}')
#    print('  mu =',fpar[0])
#    return(fpar)
# 
#def sigma2(aire,muestra,sep,dpatron,r2,mur):
#    """ definido para mediciones de GCC sin header en el CSV """
#    def funz1r2(x,a,b):
#        bo=bobina.bo
#        r1=bo[0]
#        dh=bo[2]
#        N=bo[3]
#        z1=a
#        bob=[r1,r2,dh,N,z1,1]
#        return dztheo.dzHF(x,bob,b,mur,3000).imag/x0    
#    dzucorr=solartron.dzcorr2(aire,muestra,sep)
#    za=solartron.mean2(solartron.read2(aire,sep))
#    z0=dzucorr[1]
#    x0=dzucorr[1].imag
#    r0=dzucorr[1].real
#    w=za[3]
#    f=za[0]
#    l0=dzucorr[1].imag/w
#    xmeas=f
#    ymeas=dzucorr[2].imag/x0
#    bo=bobina.bo
#    r1=bo[0]
#    dh=bo[2]
#    N=bo[3]
#    fpar, fcov=optimize.curve_fit(funz1r2, xmeas, ymeas, p0=[bo[1],1e6], bounds=((0,5e-3),1e6,1e8))
#    fpar
#    r2eff=r2
#    z1eff=fpar[0]
#    boeff=[r1,r2eff,dh,N,z1eff,l0]
#    sigmaeff=fpar[1]
#    yteo=dztheo.dzD(f,boeff,sigmaeff,dpatron,mur,3000)
#    yteo=yteo.imag/x0
#    semilogfit(f,[ymeas, yteo],'im(dzucorr vs dzteo) curve_fit{z1,r2}')
#    print('z1 =',fpar[0],'  sigma =',fpar[1])
#    
######### nuevo 22.02.19
#
#
#
#
#
#
#def z1r1(bo,datacorr,dpatron,sigma,mur):
#    """ fiteo z1 y r1 """
#    def funz1r1(x,b,a):
#        r1=a
#        r2=bo[1]
#        dh=bo[2]
#        N=bo[3]
#        z1=b
#        bob=[r1,r2,dh,N,z1,1]
#        return dztheo.dzD(x,bob,sigma,dpatron,mur,3000).imag/x0
#  #[f,z0,dzucorr,w]
#    f=datacorr[0]
#    l0=datacorr[1]
#    dzucorrnorm=datacorr[2]
#    w=2*np.pi*f
#    x0=w*l0
#      
#    xmeas=f
#    ymeas=dzucorrnorm.imag   
#    fpar, fcov=optimize.curve_fit(funz1r1, xmeas, ymeas, p0=[1.1e-3, bo[0]], bounds=(0,5e-3))
#    
#
#    z1eff=fpar[0]
#    r2=bo[1]
#    r1eff=fpar[1]
#    dh=bo[2]
#    N=bo[3]
#    boeff=[r1eff,r2,dh,N,z1eff,l0]
#    yteo=dztheo.dzD(f,boeff,sigma,dpatron,mur,3000)
#    yteo=yteo.imag/x0
#    
#    fig=semilogfit(f,[ymeas, yteo],'curve_fit{z1 r1}')
#    print('z1 =',fpar[0]*1000,'mm')
#    print('r1 =',fpar[1]*1000,'mm')
#
#    return([fpar,fig])
#
#def z1r2(bo,datacorr,dpatron,sigma,mur):
#    """ fiteo z1 y r2 """
#    def funz1r2(x,b,a):
#        r1=bo[0]
#        r2=a
#        dh=bo[2]
#        N=bo[3]
#        z1=b
#        bob=[r1,r2,dh,N,z1,1]
#        return dztheo.dzD(x,bob,sigma,dpatron,mur,3000).imag/x0
#    #[f,z0,dzucorr,w]
#    f=datacorr[0]
#    l0=datacorr[1]
#    dzucorrnorm=datacorr[2]
#    w=2*np.pi*f
#    x0=w*l0
#    
#    xmeas=f
#    ymeas=dzucorrnorm.imag   
#    fpar, fcov=optimize.curve_fit(funz1r2, xmeas, ymeas, p0=[1.1e-3, bo[0]], bounds=((0,3e-3),(2e-3,8e-3)))
#    
#
#    z1eff=fpar[0]
#    r1=bo[0]
#    r2eff=fpar[1]
#    dh=bo[2]
#    N=bo[3]
#    boeff=[r1,r2eff,dh,N,z1eff,l0]
#    yteo=dztheo.dzD(f,boeff,sigma,dpatron,mur,3000)
#    yteo=yteo.imag/x0
#    
#    fig=semilogfit(f,[ymeas, yteo],'curve_fit{z1 r2}')
#    print('z1 =',fpar[0]*1000,'mm')
#    print('r2 =',fpar[1]*1000,'mm')
#
#    return([fpar,fig])
#
#def z1N(bo,datacorr,dpatron,sigma,mur):
#    """" fiteo z1 y N """
#    def funz1N(x,b,a):
#        r1=bo[0]
#        r2=bo[1]
#        dh=bo[2]
#        N=a
#        z1=b
#        bob=[r1,r2,dh,N,z1,1]
#        return dztheo.dzD(x,bob,sigma,dpatron,mur,3000).imag/x0
#    #[f,z0,dzucorr,w]
#    f=datacorr[0]
#    l0=datacorr[1]
#    dzucorrnorm=datacorr[2]
#    w=2*np.pi*f
#    x0=w*l0
#    
#    xmeas=f
#    ymeas=dzucorrnorm.imag   
#    fpar, fcov=optimize.curve_fit(funz1N, xmeas, ymeas, p0=[1.1e-3, bo[3]], bounds=((0,200), (2e-3, 400)))
#    
#
#    z1eff=fpar[0]
#    r2=bo[1]
#    r1=bo[0]
#    dh=bo[2]
#    Neff=fpar[1]
#    boeff=[r1,r2,dh,Neff,z1eff,l0]
#    yteo=dztheo.dzD(f,boeff,sigma,dpatron,mur,3000)
#    yteo=yteo.imag/x0
#    
#    fig=semilogfit(f,[ymeas, yteo],'curve_fit{z1 N}')
#    print('z1 =',fpar[0]*1000,'mm')
#    print('N =',fpar[1],'vueltas')
#    return([fpar,fig])
#
#
#def bob(bo,datacorr,dpatron,sigma,mur):
#    def funz1(x,a,b,c):
#        r1=bo[0]
#        r2=a
#        dh=bo[2]
#        N=b
#        z1=c
#        bob=[r1,r2,dh,N,z1,1]
#        return dztheo.dzD(x,bob,sigma,dpatron,mur,3000).imag/x0
#    #[f,z0,dzucorr,w]
#    f=datacorr[0]
#    l0=datacorr[1]
#    dzucorrnorm=datacorr[2]
#    w=2*np.pi*f
#    x0=w*l0
#      
#    xmeas=f
#    ymeas=dzucorrnorm.imag   
#    fpar, fcov=optimize.curve_fit(funz1, xmeas, ymeas, p0=[bo[1],bo[3],bo[4]], bounds=((bo[0],240,1e-3),(5e-3,260,1.3e-3)),method='trf')
#    
#
#    z1eff=fpar[2]
#    r1=bo[0]
#    r2=fpar[0]
#    dh=bo[2]
#    N=fpar[1]
#    boeff=[r1,r2,dh,N,z1eff,l0]
#    yteo=dztheo.dzD(f,boeff,sigma,dpatron,mur,3000)
#    yteo=yteo.imag/x0
#    fig=semilogfit(f,[ymeas, yteo],'curve_fit{r2 N z1} ')
#    print(fpar)
#    return([fpar, fig])
#
### 02-19
#
#def z1mu(f,bo,datacorr,dpatron,sigma):
#    """ fiteo de z1 y mu """    
#    def funz1mu(x,a,b):
#        r1=bo[0]
#        r2=bo[1]
#        dh=bo[2]
#        N=bo[3]
#        z1=a
#        bob=[r1,r2,dh,N,z1,1]
#        return theo.dzD(x,bob,sigma,dpatron,b,3000).imag/x0
#    #[f,z0,dzucorr,w]
#    l0=bo[-1]
#    dzucorrnorm=datacorr
#    w=2*np.pi*f
#    x0=w*l0
#    
#    xmeas=f
#    ymeas=dzucorrnorm.imag   
#    fpar, fcov=optimize.curve_fit(funz1mu, xmeas, ymeas, p0=[1.1e-3, 5], bounds=((0,1), (3.5e-3, 100)))
#    
#
#    z1eff=fpar[0]
#    r2=bo[1]
#    r1=bo[0]
#    dh=bo[2]
#    Neff=bo[3]
#    boeff=[r1,r2,dh,Neff,z1eff,l0]
#    yteo=theo.dzD(f,boeff,sigma,dpatron,fpar[1],3000)
#    yteo=yteo.imag/x0
#    
#    fig=semilogfit(f,[ymeas, yteo],str(np.round(fpar[1]))+'   '+str(np.round(fpar[0]*1e3,2))+'mm')
#    print('z1 =',fpar[0]*1000,'mm')
#    print('mu =',fpar[1])
#    return([fpar,fig])
#
##### 
#def mu3(bo,datacorr,sigmas,mur2,z1,title):    
#    """propongo una region de espesor 'd' con permeablidad relativa 'mu1'"""
#    def fun(x,a,b):
#        r1=bo[0]
#        r2=bo[1]
#        dh=bo[2]
#        N=bo[3]
#        bob=[r1,r2,dh,N,z1,1]
#        return dztheo.dz2(x,bob,sigmas,a,b,mur2,3000).imag/x0      
#    #[f,z0,dzucorr,w]
#    f=datacorr[0]
#    l0=datacorr[1]
#    dzucorrnorm=datacorr[2]
#    w=2*np.pi*f
#    x0=w*l0
#      
#    xmeas=f
#    ymeas=dzucorrnorm.imag   
#    fpar, fcov=optimize.curve_fit(fun, xmeas, ymeas, p0=[0.1e-3,mur2], bounds=((0,2e-3),(1,200)))
#    
#
#    r1=bo[0]
#    r2=bo[1]
#    dh=bo[2]
#    N=bo[3]
#    boeff=[r1,r2,dh,N,z1,l0]
#    yteo=dztheo.dz2(f,boeff,sigmas,fpar[0],fpar[1],mur2,3000)
#    yteo=yteo.imag/x0
#    
#    fig=mufit.semilogfit(f,[ymeas, yteo],'curve_fit{z1} '+title)
#    print('d =',fpar[0]*1000,'mm')
#    print('mu1 =',fpar[1])
#    return(fpar,fig)



""" Modulo con funciones para graficar en matplotlin o plotly"""	
import iamend_ci.theo as theo															
import os
import csv
import numpy
import numpy as np
import cmath
import plotly.graph_objs as go
import plotly
plotly.offline.init_notebook_mode(connected=True)
import tkinter as tk
from tkinter import filedialog
import random
import matplotlib.pyplot as plt
plt.ion()
import mpld3
mpld3.enable_notebook()
from mpld3 import plugins






## matplotlib






def im(f,datacorr,n):
    """im( frecuencia, datacorr, n)
    Grafica la parte imaginaria de la impedancia corregida y normalizada.
    Parameters
    ----------
    f : array_like, vector con las frecuencias
    datacorr : array_like, matriz con las mediciones
    n : int, indice de la medicion 
    """    
    dz=datacorr[0][n]
    plt.semilogx(f,dz.imag,'ok',markersize=3,markerfacecolor='none')
    plt.ylabel('$Im(\Delta Z)/X_0$')
    plt.xlabel('Frecuencia [Hz]')
    plt.title(datacorr[1][n])
    plt.grid(True, which="both")
    
    
def re(f,datacorr,n):
    """re(frecuencia, datacorr, n)
    Grafica la parte real de la impedancia corregida y normalizada.
    Parameters
    ----------
    f : array_like, vector con las frecuencias
    datacorr : array_like, matriz con las mediciones
    n : int, indice de la medicion 
    """    
    dz=datacorr[0][n]
    plt.semilogx(f,dz.real,'ok',markersize=3,markerfacecolor='none')
    plt.ylabel('$Re(\Delta Z)/X_0$')
    plt.xlabel('Frecuencia [Hz]')
    plt.title(datacorr[1][n])
    plt.grid(True, which="both")
        


def mu(data,acero,savefile=0):
    """ agarra data procesada por fit.mu y guarda png """
    f=data[1]
    ymeas=data[2]
    mu=data[5]
    rsqr=data[6]
    yteo=data[-2]
    name=data[-1]
    plt.figure(figsize=(7,5))
    plt.semilogx(f,ymeas.imag,'ok',markersize=4,markerfacecolor='none')
    plt.semilogx(f,yteo,'k',label='$\mu_r$ = '+ str(np.round(mu,2)) + '   $r^2$ = ' + str(np.round(rsqr,3)  ))
    plt.ylabel('$Im(\Delta Z)/X_0$',fontsize=12)
    plt.xlabel('Frecuencia [Hz]',fontsize=12)
    plt.legend(loc='lower left', prop={'size': 13})
    #plt.title(acero+name[4:])
    plt.title(acero)
    plt.grid(True, which="both")
    if savefile==1:
        g=name.split(' ')
        fname=''.join(g)
        plt.savefig(fname)       
        
        
def sigma(data,savefile=0):
    """ agarra data procesada por fit.sigma y guarda png """
    f=data[0]
    ymeas=data[1]
    mu=data[3]
    rsqr=data[4]
    yteo=data[2]
    plt.figure()
    plt.semilogx(f,ymeas,'ok',markersize=4,markerfacecolor='none')
    plt.semilogx(f,yteo,'k',label='$\sigma$ = '+ str(np.round(mu,2)) + '   $r^2$ = ' + str(np.round(rsqr,3)  ))
    plt.ylabel('$Im(\Delta Z)/X_0$')
    plt.xlabel('Frecuencia [Hz]')
    plt.legend(loc='lower left')
   
     


def fmu(fdata,save=0,name='default'):
    """ agarra data procesada por fit.fmu y guarda png """
    mrks=['o','s','p','^','*','X']
    plt.figure(figsize=(7,5))
    for i,x in enumerate(fdata[0]):
        f=fdata[1][i]
        ymeas=fdata[2][i]
        yteo=fdata[3][i]
        plt.semilogx(f,ymeas,mrks[i]+'k',markersize=5,markerfacecolor='none',label='mu = '+ str(np.round(fdata[0][i],3)) )
        plt.semilogx(f,yteo,'-k')
    plt.ylabel('$Im(\Delta Z)/X_0$',fontsize=12)
    plt.xlabel('Frecuencia [Hz]',fontsize=12)
    plt.legend(loc='lower left', prop={'size': 13})
    plt.grid(True, which="both")
    if name=='default':
        plt.title(fdata[-1])
    else:
        plt.title(name)
    if save==1:
        g=fdata[-1].split(' ')
        fname=''.join(g)
        plt.savefig('fmu_'+fname)


        
def ffit(fdata,save=0,fit='par',name=''):
    """ agarra data procesada por fit.ffmu guarda png """
    mrks=['o','s','p','^','*','X']
    plt.figure()
    for i,x in enumerate(fdata[0]):
        f=fdata[1][i]
        ymeas=fdata[2][i]
        yteo=fdata[3][i]
        plt.semilogx(f,ymeas,mrks[i]+'k',markersize=5,markerfacecolor='none',label=fit+' = '+ str(np.round(fdata[0][i],3)) )
        plt.semilogx(f,yteo,'-k')
    plt.ylabel('$Im(\Delta Z)/X_0$')
    plt.xlabel('Frecuencia [Hz]')
    plt.legend(loc='lower left')
    plt.title(name)
    if save==1:
        g=fdata[-1].split(' ')
        fname=''.join(g)
        plt.savefig('fmu_'+fname)
        

        
        
## plotly

def semilog(x,datacorr,titulo='',mode='lines'):
    Y=datacorr[0].imag
    data=[]
    for b,a in enumerate(Y):
        trace0 = go.Scatter(
        x = x,
        y = a,
        name=datacorr[1][b],
        mode = mode)
        data.append(trace0)

    layout = go.Layout(
        xaxis=dict(        type='log',zeroline=False , autorange=True   ),
        yaxis=dict(        type='linear',zeroline=False,autorange=True  ),
        title=titulo
    )

    fig = go.Figure(data=data, layout=layout)
    plotly.offline.iplot(fig)
    return(fig)




def ims(x,datacorr,titulo='',mode='lines'):
    data=[]
    Y=datacorr[0].imag
    datas=datacorr[1]
    for b,a in enumerate(Y):
        trace0 = go.Scatter(
        x = x,
        y = a,
        name=datas[b],    
        mode = mode)
        data.append(trace0)

    layout = go.Layout(
        xaxis=dict(        type='log',zeroline=False , autorange=True   ),
        yaxis=dict(        type='linear',zeroline=False,autorange=True  ),
        title=titulo
    )

    fig = go.Figure(data=data, layout=layout)
    plotly.offline.iplot(fig)



def res(x,datacorr,titulo='',mode='lines'):
    data=[]
    Y=datacorr[0].real
    datas=datacorr[1]
    for b,a in enumerate(Y):
        trace0 = go.Scatter(
        x = x,
        y = a,
        name=datas[b],    
        mode = mode)
        data.append(trace0)

    layout = go.Layout(
        xaxis=dict(        type='log',zeroline=False , autorange=True   ),
        yaxis=dict(        type='linear',zeroline=False,autorange=True  ),
        title=titulo
    )

    fig = go.Figure(data=data, layout=layout)
    plotly.offline.iplot(fig)




def rawplot(dataraw,m):
    data=dataraw[0][m]
    i=int(data[1][-1])
    n=int(len(data[3])/i)
    x=data[2][0:n]
    reY=np.reshape(data[3],(i,n))
    imY=np.reshape(data[4]/data[5],(i,n))
    nms=list()
    for j in np.arange(0,i): nms.append(str(j))
    res(x,list([reY,nms]),titulo='Re(z)'+dataraw[1][m])
    res(x,list([imY,nms]),titulo='Im(z)'+dataraw[1][m])
    



#def Y(data,f='null',mode='lines',type='linear'):
#    """ agarra matriz"""
#    if f=='null':
#        x=np.arange(1,data.shape[0]+1)
#    else:        
#        x=f 
#    Y=data
#    dataplots=list()
#    for i in range(0,Y.shape[1]):
#        trace = go.Scatter(
#        x = x,
#        y = Y[:,i],
#        mode = mode)
#        dataplots.append(trace)
#
#    layout = go.Layout(
#        xaxis=dict(        type=type,zeroline=False , autorange=True   ),
#        yaxis=dict(        type='linear',zeroline=False,autorange=True  )
#            )
#
#    figs = go.Figure(data=dataplots, layout=layout)
#    return(figs)       												
#
#def gety(fig,err=0):
#    if err==0:
#        ys=list()
#        for x in fig.data:
#            ys.append(x['y'])
#    else:
#        ys=list()
#        
#        for i,x in enumerate(fig.data):
#            ys.append(fig.data[i*3+3]['y'])    
#        
#    return(np.array(ys))    
#
#def u(*args,mode='lines',type='linear'):
#    X=args[0::2]
#    Y=args[1::2]
#    dataplots=list()
#    for i,x in enumerate(X):
#        trace = go.Scatter(
#        x = x,
#        y = Y[i],
#        mode = mode)
#        dataplots.append(trace)
#
#    layout = go.Layout(
#        xaxis=dict(        type=type,zeroline=False , autorange=True   ),
#        yaxis=dict(        type=type,zeroline=False,autorange=True  )
#            )
#
#    figs = go.Figure(data=dataplots, layout=layout)
#    return(figs)	
#
#def v(*args,mode='lines',type='linear'):
#    X=args[0::2]
#    Y=args[1::2]
#    dataplots=list()
#    for i,x in enumerate(X):
#        trace = go.Scatter(
#        x = x,
#        y = Y[i],
#        mode = mode)
#        dataplots.append(trace)
#
#    layout = go.Layout(
#        xaxis=dict(        type=type,zeroline=False , autorange=True   ),
#        yaxis=dict(        type=type,zeroline=False,autorange=True  )
#            )
#
#    figs = go.Figure(data=dataplots, layout=layout)
#    plotly.offline.iplot(figs)
#    return(figs)    
#
#
#
#
#
#def logx(*args):
#    X=args[0::2]
#    Y=args[1::2]
#    dataplots=list()
#    for i,x in enumerate(X):
#        trace = go.Scatter(
#        x = x,
#        y = Y[i],
#        mode = 'lines')
#        dataplots.append(trace)
#
#    layout = go.Layout(
#        xaxis=dict(        type='log',zeroline=False , autorange=True   ),
#        yaxis=dict(        type='linear',zeroline=False,autorange=True  )
#            )
#
#    figs = go.Figure(data=dataplots, layout=layout)
#    plotly.offline.iplot(figs)
#    return(figs)
#
#
#
#def vs(X,Y,tipo='log'):
#    dataplots=[]
#    for i,x in enumerate(X):
#        trace0 = go.Scatter(
#                x = x,
#                y = Y[i],
#                mode = 'lines')
#        dataplots.append(trace0)
#    layout = go.Layout(
#                xaxis=dict(        type=tipo,zeroline=False , autorange=True   ),
#                yaxis=dict(        type='linear',zeroline=False,autorange=True  )            )
#    
#    fig = go.Figure(data=dataplots, layout=layout)
#    plotly.offline.iplot(fig)
#    return(fig)
#
#def cat(*args):
#    Fig=getfig(tipo='linear')
#    if len(args)==1:
#        figs=args[0]
#        for i in range(0,len(figs)):
#            Fig.add_traces(figs[i].data[:])
#        ret=Fig    
#    else:
#        
#        for i in range(0,len(args)):
#            Fig.add_traces(args[i].data[:])
#        ret=Fig   
#    return(ret)    
#
#
#def getfig(tipo='log'):
#    layout = go.Layout(
#        xaxis=dict(        type=tipo,zeroline=False , autorange=True   ),
#        yaxis=dict(        type='linear',zeroline=False,autorange=True  )
#    )
#    fig = go.Figure(layout=layout)
#    return(fig)
#    
#
#
#
#    
#    
#def semilogn(x,Y):
#    data=[]
#    for b,a in enumerate(Y):
#        trace0 = go.Scatter(
#        x = x,
#        y = a/(2*np.pi*x),
#        mode = 'markers')
#        data.append(trace0)
#
#    layout = go.Layout(
#        xaxis=dict(        type='log',zeroline=False , autorange=True   ),
#        yaxis=dict(        type='linear',zeroline=False,autorange=True  )
#    )
#
#    fig = go.Figure(data=data, layout=layout)
#    plotly.offline.iplot(fig)
#    return(fig)
#
#
#
#def semilogm(x,Y,titulo=''):
#    data=[]
#    for b,a in enumerate(Y):
#        trace0 = go.Scatter(
#        x = x,
#        y = a,
#        mode = 'markers')
#        data.append(trace0)
#    Ymean=numpy.mean(Y,0)
#    trace0 = go.Scatter(
#    x = x,
#    y = Ymean)
#    data.append(trace0)
#
#    layout = go.Layout(
#        xaxis=dict(        type='log',zeroline=False , autorange=True   ),
#        yaxis=dict(        type='linear',zeroline=False,autorange=True  ),
#        title=titulo
#    )
#
#
#    plotly.offline.iplot(fig)  
#
#
#
    

#### solartron



#
#
#def irawplot(datax,p):
#    data=datax[0][p]
#    i=data[1][-1]
#    n=int(len(data[3])/i)
#    x=data[2][0:n]
#    imY=np.reshape(data[4]/data[5],(i,n))
#    fig=semilog(x,imY,datax[1][p]+'  Im(Z)')
#    return(fig)
#
#
#def iallraw(DATA):
#    figs=list()
#    for p,x in enumerate(DATA[0]):
#        data=x
#        i=int(data[1][-1])
#        n=int(len(data[3])/i)
#        x=data[2][0:n]
#        imY=np.reshape(data[4]/data[5],(i,n))
#        fig=semilog(x,imY,DATA[1][p]+'  Im(Z)')
#        figs.append(fig)
#    return(figs)
#    
#def solarplots(file):
#    """ procesa un archivo, solo arma la matriz """
#    data1=read(file,';')
#    #  return(n,i,f, rez,imz, w )
#    N=len(data1[0])
#    print('N='+str(len(data1[0])))
#    r=data1[1][N-1]
#    print('#iteraciones='+str(data1[1][N-1]))
#    n=int(N/data1[1][N-1])
#    print('#frecuencias='+str(n))
#    semilogm(data1[2][0:n-1],(data1[4]/data1[5]).reshape((r,n)),file)
#    #promedia las N mediciones
#    #datam1=solartron.mean(data1)
#    # rertunr ([fs,rezum,imzum,ws])
#    #semilog(datam1[0],[datam1[2]/datam1[3]],'Promedio de las N iteraciones de una medicion')
#    
#    
#
#    
#def meanplot(data,title):
#    i=data[1][-1]
#    n=int(len(data[3])/i)
#    x=data[2][0:n]
#    reY=np.reshape(data[3],(i,n))
#    imY=np.reshape(data[4]/data[5],(i,n))
#    semilogm(x,reY,title+'  Re(Z)')
#    semilogm(x,imY,title+'  Im(Z)')

###### plots    



def errorplotvR(data,v):
    data2=np.array(data)
    dataz3=data2[0][v]
    DATA=[]
    for n,x in enumerate(dataz3):
        ni=x[1][-1]
        nf=int(int(x[0][-1])/ni)
        f=np.reshape(x[2],(ni,nf))
        fm=np.mean(f,0)
        X=np.reshape(x[3],(ni,nf))
        X=np.array(X)
        X=np.array(X[1:,:]) #saco la primer iteracion
        xm=np.mean(X,0)
        err=list()
        for i in range(0,len(X[0,:])):
            y=X[:,i]
            err.append((max(y)-min(y))/2)
        err=np.array(err)
        upper_bound = go.Scatter(
            name='Upper Bound',
            x=fm,
            y=xm+err/2,
            mode='lines',
            marker=dict(color="#444"),
            line=dict(width=0),
            fillcolor='rgba(68, 68, 68, 0.3)',
            fill='tonexty',
            showlegend=False,
            legendgroup=data2[1][v[n]])
        
        trace = go.Scatter(
            name=data2[1][v[n]],
            x=fm,
            y=xm,
            mode='lines',
            #line=dict(color='rgb(31, 119, 180)'),
            fillcolor='rgba(68, 68, 68, 0.3)',
            fill='tonexty',
            legendgroup=data2[1][v[n]])

        lower_bound = go.Scatter(
            name='Lower Bound',
            x=fm,
            y=xm-err/2,
            marker=dict(color="#444"),
            line=dict(width=0),
            mode='lines',
            showlegend=False,
            legendgroup=data2[1][v[n]])

        # Trace order can be important
        # with continuous error bars
        data = [lower_bound, trace, upper_bound]

        #layout = go.Layout(
        #    yaxis=dict(title='Wind speed (m/s)'),
        #    showlegend = False)

        DATA.extend(data)

    layout = go.Layout(
            xaxis=dict(        type='log',zeroline=False , autorange=True   ),
            yaxis=dict(        type='linear',zeroline=False,autorange=True  ),
            )    
    fig = go.Figure(data=DATA, layout=layout)
    plotly.offline.iplot(fig)
	
def imag(data,m):
    dataz=data[0][m]
    ni=dataz[1][-1]
    nf=int(int(dataz[0][-1])/ni)
    f=np.reshape(dataz[2],(ni,nf))
    fm=np.mean(f,0)
    X=np.reshape(dataz[4]/dataz[5],(ni,nf))
    X=np.array(X)
    xm=np.mean(X,0)
    err=list()
    for i in range(0,len(X[0,:])):
        y=X[:,i]
        err.append((max(y)-min(y))/2)
    err=np.array(err)

    upper_bound = go.Scatter(
        name='Upper Bound',
        x=fm,
        y=xm+err/2,
        mode='lines',
        marker=dict(color="#444"),
        line=dict(width=0),
        fillcolor='rgba(68, 68, 68, 0.3)',
        fill='tonexty',
        showlegend=False,
        legendgroup=data[1][m])

    trace = go.Scatter(
        x=fm,
        y=xm,
        mode='lines',
        name=data[1][m],
        legendgroup=data[1][m],
        fillcolor='rgba(68, 68, 68, 0.3)',
        fill='tonexty')

    lower_bound = go.Scatter(
        name='Lower Bound',
        x=fm,
        y=xm-err/2,
        marker=dict(color="#444"),
        line=dict(width=0),
        mode='lines',
        showlegend=False,
        legendgroup=data[1][m])

    # Trace order can be important
    # with continuous error bars
    data = [lower_bound, trace, upper_bound]

    #layout = go.Layout(
    #    yaxis=dict(title='Wind speed (m/s)'),
    #    showlegend = False)
    layout = go.Layout(
            xaxis=dict(        type='log',zeroline=False , autorange=True   ),
            yaxis=dict(        type='linear',zeroline=False,autorange=True  ) )


    fig = go.Figure(data=data, layout=layout)
      
    return(fig)


def errorplotvL(data,v):
    data2=np.array(data)
    dataz3=data2[0][v]
    DATA=[]
    for n,x in enumerate(dataz3):
        ni=x[1][-1]
        nf=int(int(x[0][-1])/ni)
        f=np.reshape(x[2],(ni,nf))
        fm=np.mean(f,0)
        X=np.reshape(x[4]/x[5],(ni,nf))
        X=np.array(X)
        X=np.array(X[1:,:]) #saco la primer iteracion
        xm=np.mean(X,0)
        err=list()
        for i in range(0,len(X[0,:])):
            y=X[:,i]
            err.append((max(y)-min(y))/2)
        err=np.array(err)
        upper_bound = go.Scatter(
            name='Upper Bound',
            x=fm,
            y=xm+err/2,
            mode='lines',
            marker=dict(color="#444"),
            line=dict(width=0),
            fillcolor='rgba(68, 68, 68, 0.3)',
            fill='tonexty',
            showlegend=False,
            legendgroup=data2[1][v[n]])
        
        trace = go.Scatter(
            name=data2[1][v[n]],
            x=fm,
            y=xm,
            mode='lines',
            #line=dict(color='rgb(31, 119, 180)'),
            fillcolor='rgba(68, 68, 68, 0.3)',
            fill='tonexty',
            legendgroup=data2[1][v[n]])

        lower_bound = go.Scatter(
            name='Lower Bound',
            x=fm,
            y=xm-err/2,
            marker=dict(color="#444"),
            line=dict(width=0),
            mode='lines',
            showlegend=False,
            legendgroup=data2[1][v[n]])

      
    fig = plotly.tools.make_subplots(rows=1, cols=1 ,print_grid=False)
    fig.append_trace(lower_bound, 1, 1)
    fig.append_trace(trace, 1, 1)
    fig.append_trace(upper_bound, 1, 1)
    fig["layout"]["xaxis"].update( type='log',zeroline=False , autorange=True   )
    fig["layout"]["yaxis"].update( type='linear',zeroline=False , autorange=True   )

    
    return(fig)    	


def errplot(data,n):
    rr=random.randint(0,12)
    ns=2
    x=data[0][n][2][0:35]
    zim=data[0][n][4].reshape(11,35)
    y=np.mean(zim,0)/(2*np.pi*x)
    y_up=(np.mean(zim,0)+ns*np.std(zim,0))/(2*np.pi*x)
    y_lw=(np.mean(zim,0)-ns*np.std(zim,0))/(2*np.pi*x)

    upper_bound = go.Scatter(
        name='Upper Bound',
        x=x,
        y=y_up,
        mode='lines',
        line=dict(width=0),
        fill='tonexty',
        fillcolor=lrgb2a[rr],
            showlegend=False,
        legendgroup=data[1][n])

    trace = go.Scatter(
        
        x=x,
        y=y,
        mode='markers+lines',
        
        line=dict(width=0.9),
        marker={'symbol':'x','color':lrgb2[rr]},  
        fill='tonexty',
        fillcolor=lrgb2a[rr],
        name=data[1][n],
        showlegend=True,
        legendgroup=data[1][n])


    lower_bound = go.Scatter(
        name='Lower Bound',
        x=x,
        y=y_lw,      
        line=dict(width=0),
        mode='lines',
            showlegend=False,
        legendgroup=data[1][n])


    # Trace order can be important
    # with continuous error bars
    dataplot = [lower_bound, trace, upper_bound]

    #layout = go.Layout(
    #    yaxis=dict(title='Wind speed (m/s)'),
    #    showlegend = False)
    layout = go.Layout(
            xaxis=dict(        type='log',zeroline=False , autorange=True   ),
            yaxis=dict(        type='linear',zeroline=False,autorange=True  ),
    
        )

    fig = go.Figure(data=dataplot, layout=layout)
    return(fig)




def errplotall(dataraw,corr=0,Z0=0):
    if corr==1:
        figs=list()
        data=[datamean(dataraw),dataraw[1]]
        datas=data[1]
        data=np.array(data[0])
        za=data[0][1]
        
        datacorr=[]
        lwerr=[]
        uperr=[]
        for i,x in enumerate(data):
                    rr=random.randint(0,11)
                    f=x[0].real
                    zu=x[1]
                    w=x[2].real
                    xraw=dataraw[0][i][4].reshape(int(dataraw[0][i][4].shape[0]/f.shape[0]),f.shape[0])
                    x0=w*Z0.imag
                    z0=Z0.real+1j*x0

                    ns=2    
                    dzucorr=((1/(1/zu - 1/za + 1/z0))-z0  )
                    yupcorr=((1/(1/(zu+1j*ns*np.std(xraw,0)) - 1/za + 1/z0))-z0  )
                    ylwcorr=((1/(1/(zu-1j*ns*np.std(xraw,0)) - 1/za + 1/z0))-z0  )


                    y = (np.imag(dzucorr)/x0).real
                    y_upcorr=(np.imag(yupcorr)/x0).real
                    y_lwcorr=(np.imag(ylwcorr)/x0).real
                    
                           
                    trace0 = go.Scatter(
                    x =f ,
                    y = y,
                    mode = 'markers+lines',
                    line=dict(width=0.9),
                    marker={'symbol':'x','color':lrgb2[rr]},  
                    fill='tonexty',
                    fillcolor=lrgb2a[rr],
                    name=datas[i],
                    showlegend=True,
                    legendgroup=datas[i])
                    
                    upper_bound = go.Scatter(
                        name='Upper Bound',
                        x=f,
                        y=y_upcorr,
                        mode='lines',
                        line=dict(width=0),
                        fill='tonexty',
                        fillcolor=lrgb2a[rr],
                            showlegend=False,
                        legendgroup=datas[i])                    

                    lower_bound = go.Scatter(
                        name='Lower Bound',
                        x=f,
                        y=y_lwcorr,      
                        line=dict(width=0),
                        mode='lines',
                            showlegend=False,
                        legendgroup=datas[i])
                   
                    
                    datacorr.append(dzucorr/x0)    
                    lwerr.append(ylwcorr/x0)           
                    uperr.append(yupcorr/x0) 
                    
                    
                    layout = go.Layout(
                    xaxis=dict(        type='log',zeroline=False , autorange=True   ),
                    yaxis=dict(        type='linear',zeroline=False,autorange=True  ))
                    figs.append(go.Figure(data=[lower_bound, trace0, upper_bound], layout=layout))

        for i in range(1,len(figs)):
            figs[0].add_traces(figs[i].data)
            
        ret=[np.array(datacorr[1:]),np.array(lwerr[1:]),np.array(uperr[1:]),figs[0]]
        
    else:
        figs=list()
        data=dataraw
        for n,datax in enumerate(data[0]):
            rr=random.randint(0,11)
            ni=data[0][n][1][-1]
            nf=int(int(data[0][n][0][-1])/ni)

            ns=2
            x=data[0][n][2][0:nf]
            zim=data[0][n][4].reshape(ni,nf)
            y=np.mean(zim,0)/(2*np.pi*x)
            y_up=(np.mean(zim,0)+ns*np.std(zim,0))/(2*np.pi*x)
            y_lw=(np.mean(zim,0)-ns*np.std(zim,0))/(2*np.pi*x)

            upper_bound = go.Scatter(
                name='Upper Bound',
                x=x,
                y=y_up,
                mode='lines',
                line=dict(width=0),
                fill='tonexty',
                fillcolor=lrgb2a[rr],
                    showlegend=False,
                legendgroup=data[1][n])

            trace = go.Scatter(

                x=x,
                y=y,
                mode='markers+lines',

                line=dict(width=0.9),
                marker={'symbol':'x','color':lrgb2[rr]},  
                fill='tonexty',
                fillcolor=lrgb2a[rr],
                name=data[1][n],
                showlegend=True,
                legendgroup=data[1][n])


            lower_bound = go.Scatter(
                name='Lower Bound',
                x=x,
                y=y_lw,      
                line=dict(width=0),
                mode='lines',
                    showlegend=False,
                legendgroup=data[1][n])


            # Trace order can be important
            # with continuous error bars
            dataplot = [lower_bound, trace, upper_bound]

            #layout = go.Layout(
            #    yaxis=dict(title='Wind speed (m/s)'),
            #    showlegend = False)
            layout = go.Layout(
                    xaxis=dict(        type='log',zeroline=False , autorange=True   ),
                    yaxis=dict(        type='linear',zeroline=False,autorange=True  ),

                )

            figs.append(go.Figure(data=dataplot, layout=layout))

        for i in range(1,len(figs)):
            figs[0].add_traces(figs[i].data)
        ret=figs[0]    
    return(ret)

def errplotre(dataz):
    ni=dataz[1][-1]
    nf=int(int(dataz[0][-1])/ni)
    f=np.reshape(dataz[2],(ni,nf))
    fm=np.mean(f,0)
    X=np.reshape(dataz[3],(ni,nf))
    X=np.array(X)
    xm=np.mean(X,0)
    err=list()
    for i in range(0,len(X[0,:])):
        y=X[:,i]
        err.append((max(y)-min(y))/2)
    err=np.array(err)
    
    
    
    upper_bound = go.Scatter(
        name='Upper Bound',
        x=fm,
        y=xm+err/2,
        mode='lines',
        line=dict(width=0),
        fillcolor=lrgb2a[rr],
                showlegend=False,
        fill='tonexty')

    trace = go.Scatter(
        name='Measurement',
        x=fm,
        y=xm,
        mode='lines',
        line=dict(color='rgb(31, 119, 180)'),
        fillcolor='rgba(68, 68, 68, 0.3)',
        fill='tonexty')

    lower_bound = go.Scatter(
        name='Lower Bound',
        x=fm,
        y=xm-err/2,
        marker=dict(color="#444"),
        line=dict(width=0),
        mode='lines')

    # Trace order can be important
    # with continuous error bars
    data = [lower_bound, trace, upper_bound]

    #layout = go.Layout(
    #    yaxis=dict(title='Wind speed (m/s)'),
    #    showlegend = False)
    layout = go.Layout(
            xaxis=dict(        type='log',zeroline=False , autorange=True   ),
            yaxis=dict(        type='linear',zeroline=False,autorange=True  ),
            showlegend = False,
        )
    fig = go.Figure(data=data, layout=layout)
    
    return(fig)
	
def imags(*args):
    figs=list()
    DATA=args[0::2]
    M=args[1::2]
    for i,data in enumerate(DATA):
        m=M[i]
        dataz=data[0][m]
        ni=dataz[1][-1]
        nf=int(int(dataz[0][-1])/ni)
        f=np.reshape(dataz[2],(ni,nf))
        fm=np.mean(f,0)
        X=np.reshape(dataz[4]/dataz[5],(ni,nf))
        X=np.array(X)
        xm=np.mean(X,0)
        err=list()
        for i in range(0,len(X[0,:])):
            y=X[:,i]
            err.append((max(y)-min(y))/2)
        err=np.array(err)

        upper_bound = go.Scatter(
            name='Upper Bound',
            x=fm,
            y=xm+err/2,
            mode='lines',
            marker=dict(color="#444"),
            line=dict(width=0),
            fillcolor='rgba(68, 68, 68, 0.3)',
            fill='tonexty',
            showlegend=False,
            legendgroup=data[1][m])

        trace = go.Scatter(
            x=fm,
            y=xm,
            mode='lines',
            name=data[1][m],
            legendgroup=data[1][m],
            fillcolor='rgba(68, 68, 68, 0.3)',
            fill='tonexty')

        lower_bound = go.Scatter(
            name='Lower Bound',
            x=fm,
            y=xm-err/2,
            marker=dict(color="#444"),
            line=dict(width=0),
            mode='lines',
            showlegend=False,
            legendgroup=data[1][m])

        # Trace order can be important
        # with continuous error bars
        data = [lower_bound, trace, upper_bound]

        #layout = go.Layout(
        #    yaxis=dict(title='Wind speed (m/s)'),
        #    showlegend = False)
        layout = go.Layout(
                xaxis=dict(        type='log',zeroline=False , autorange=True   ),
                yaxis=dict(        type='linear',zeroline=False,autorange=True  ) )


        fig = go.Figure(data=data, layout=layout)
        figs.append(fig)
      
    return(figs)







        
defaultsa=['rgba(31, 119, 180,0.5)',
 'rgba(255, 127, 14,0.5)',
 'rgba(44, 160, 44,0.5)',
 'rgba(214, 39, 40,0.5)',
 'rgba(148, 103, 189,0.5)',
 'rgba(140, 86, 75,0.5)',
 'rgba(227, 119, 194,0.5)',
 'rgba(127, 127, 127,0.5)',
 'rgba(188, 189, 34,0.5)',
 'rgba(23, 190, 207,0.5)']


lrgb=['rgb(141,211,199)',
 'rgb(255,255,179)',
 'rgb(190,186,218)',
 'rgb(251,128,114)',
 'rgb(128,177,211)',
 'rgb(253,180,98)',
 'rgb(179,222,105)',
 'rgb(252,205,229)',
 'rgb(217,217,217)',
 'rgb(188,128,189)',
 'rgb(204,235,197)']

lrgb2=['rgb(166,206,227)','rgb(31,120,180)','rgb(178,223,138)','rgb(51,160,44)','rgb(251,154,153)','rgb(227,26,28)','rgb(253,191,111)','rgb(255,127,0)','rgb(202,178,214)','rgb(106,61,154)','rgb(255,255,153)','rgb(177,89,40)']

lrgb2a=['rgba(166,206,227,0.5)',
 'rgba(31,120,180,0.5)',
 'rgba(178,223,138,0.5)',
 'rgba(51,160,44,0.5)',
 'rgba(251,154,153,0.5)',
 'rgba(227,26,28,0.5)',
 'rgba(253,191,111,0.5)',
 'rgba(255,127,0,0.5)',
 'rgba(202,178,214,0.5)',
 'rgba(106,61,154,0.5)',
 'rgba(255,255,153,0.5)',
 'rgba(177,89,40,0.5)']

lrgba=['rgba(141,211,199,0.5)',
 'rgba(255,255,179,0.5)',
 'rgba(190,186,218,0.5)',
 'rgba(251,128,114,0.5)',
 'rgba(128,177,211,0.5)',
 'rgba(253,180,98,0.5)',
 'rgba(179,222,105,0.5)',
 'rgba(252,205,229,0.5)',
 'rgba(217,217,217,0.5)',
 'rgba(188,128,189,0.5)',
 'rgba(204,235,197,0.5)']	





""" Modulo para el procesamiento de los archivos CVS del solartron"""
import iamend_ci.theo as theo

import os
import tkinter as tk
from tkinter import filedialog
import csv
import numpy
import numpy as np
import cmath
import plotly.graph_objs as go
import plotly
plotly.offline.init_notebook_mode(connected=True)
import random



###### OLD
	
def zload(): 
    root = tk.Tk()
    root.lift()
    root.attributes('-topmost',True)
    root.after_idle(root.attributes,'-topmost',False)
    folder_path = filedialog.askdirectory()
    files=list()
    print(folder_path)
    for (dirpath, dirnames, filenames) in os.walk(folder_path):
        
        for i,j in enumerate(filenames):
            files.extend([dirpath + '\\'+j])
            print(i,j)
        break
    
    root.withdraw()
    return (files,filenames)

def csv2data(files,separador):
    
    
    data=list()
    for file in files[0]:
        data.append(read(file,separador))
    return(data,files[1])



def read(file, separador):

    z=list()
    with open(file, 'r') as csvfile:
        spam = csv.reader(csvfile, delimiter=separador)

        next(spam)
        next(spam)
        next(spam)
        next(spam)

        for row in spam:
            z.append(row)

    f=list()
    rez=list()
    imz=list()
    n=list()
    i=list()

    for x in range(len(z)):
        f.append(float(z[x][4]))
        rez.append(float(z[x][12]))
        imz.append(float(z[x][13]))
        n.append(int(z[x][0]))
        i.append(int(z[x][1]))

    f = numpy.asarray(f)
    w=2*numpy.pi*f
    rez=numpy.asarray(rez)
    imz=numpy.asarray(imz)
    n=numpy.asarray(n)
    i=numpy.asarray(i)
    out=list([n,i,f, rez,imz, w])
    out=numpy.array(out)
    return(out)

def read2(file, separador):
    z=list()
    with open(file, 'r') as csvfile:
        spam = csv.reader(csvfile, delimiter=separador)
        for row in spam:
            z.append(row)

    f=list()
    rez=list()
    imz=list()
    n=list()
    i=list()
    for x in range(len(z)):
        f.append(float(z[x][4]))
        rez.append(float(z[x][7]))
        imz.append(float(z[x][8]))
        n.append(int(z[x][0]))
        i.append(int(z[x][1]))
    f = numpy.asarray(f)
    w=2*numpy.pi*f
    rez=numpy.asarray(rez)
    imz=numpy.asarray(imz)
    n=numpy.asarray(n)
    i=numpy.asarray(i)
    return(n,i,f, rez,imz, w )

def mean(zs):
    zs=numpy.array(zs)
    # numero de iteraciones
    ni=int(zs[1][len(zs[0])-1])
    # numero total de mediciones
    n=len(zs[1])
    # numero de puntos de cada medicion
    nm=int(n/ni)
    zsmean=list()
    f=zs[2]
    rezu=zs[3]
    imzu=zs[4]
    w=zs[5]
    rezu2=numpy.reshape(rezu,(ni,nm))
    imzu2=numpy.reshape(imzu,(ni,nm))
    # tiro la primera
    rezum=numpy.mean(rezu2[1:11],0)
    imzum=numpy.mean(imzu2[1:11],0)
    fs=numpy.reshape(f,(ni,nm))
    fs=numpy.mean(fs,0)
    ws=2*numpy.pi*fs
    zsmean=numpy.array([fs,rezum,imzum,ws])
    return(zsmean)

def mean2(zs):
    zs=numpy.array(zs)
    # numero de iteraciones
    ni=int(zs[1][len(zs[0])-1])
    # numero total de mediciones
    n=len(zs[1])
    # numero de puntos de cada medicion
    nm=int(n/ni)
    zsmean=list()
    f=zs[2]
    rezu=zs[3]
    imzu=zs[4]
    w=zs[5]
    rezu2=numpy.reshape(rezu,(ni,nm))
    imzu2=numpy.reshape(imzu,(ni,nm))
    # tiro la primera
    rezum=numpy.mean(rezu2[1:11],0)
    imzum=numpy.mean(imzu2[1:11],0)
    fs=numpy.reshape(f,(ni,nm))
    fs=numpy.mean(fs,0)
    ws=2*numpy.pi*fs
    zsmean=numpy.array([fs,rezum,imzum,ws])
    return(zsmean)

def zcorr_vf(w,za,zu,r0,l0):
    z0=r0+1j*w*l0
    return (((1/(1/zu - 1/za + 1/z0))-z0))

def z0(aire):
    data1=read(aire,';')
    data1=mean(data1)
    #  return[fs,rezum,imzum,ws]
    ws=data1[3]
    l0=(data1[2]/ws)[0]
    r0=data1[1][0]
    za=data1[1]+1j*data1[2]
    f=data1[0]
    z0=r0+1j*ws*l0
    return([f,za,z0,w])


# cargo datos del aire
def dzcorr(aire,file,sep):
    data1=read(aire,sep)
    data1=mean(data1)
    #  return[fs,rezum,imzum,ws]
    ws=data1[3]
    l0=(data1[2]/ws)[0]
    r0=data1[1][0]
    za=data1[1]+1j*data1[2]
    f=data1[0]
    z0=r0+1j*ws*l0

    ##cargo datos a corregir
    data2=read(file,sep)
    data2=mean(data2)
    zu=data2[1]+1j*data2[2]
    dzucorr=((1/(1/zu - 1/za + 1/z0))-z0)
    return([f,z0,dzucorr])


def dzcorr2(aire,file,sep):
    """ cargo datos del aire """
    data1=read2(aire,sep)
    data1=mean2(data1)
    #  return[fs,rezum,imzum,ws]
    ws=data1[3]
    l0=(data1[2]/ws)[0]
    r0=data1[1][0]
    za=data1[1]+1j*data1[2]
    f=data1[0]
    z0=r0+1j*ws*l0

    ##cargo datos a corregir
    data2=read2(file,sep)
    data2=mean2(data2)
    zu=data2[1]+1j*data2[2]
    dzucorr=((1/(1/zu - 1/za + 1/z0))-z0)
    return([f,z0,dzucorr])
        
def findnear(A,x):
    idx=numpy.argmin(numpy.abs(A - x))
    return(idx)

    


    

def meanz0(data,v):
    """ le saca la primera iteracion """
    data2=np.array(data)
    dataz3=data2[0][v]
    DATA=[]
    for n,x in enumerate(dataz3):
        ni=x[1][-1]
        nf=int(int(x[0][-1])/ni)
        X=np.reshape(x[4]/x[5],(ni,nf))
        Y=np.reshape(x[3],(ni,nf))
        Y=np.array(Y[1:,:])
        X=np.array(X[1:,:])
        xm=np.mean(np.mean(X,0))
        ym=np.mean(np.mean(Y,0))
        DATA.append(ym+1j*xm)
    return(np.array(DATA))

def datamean(data):
    dataz3=data[0]
    DATA=[]
    for n,x in enumerate(dataz3):
        ni=x[1][-1]
        nf=int(int(x[0][-1])/ni)
        X=np.reshape(x[4],(ni,nf))
        R=np.reshape(x[3],(ni,nf))
        f=np.reshape(x[2],(ni,nf))
        R=np.array(R[1:,:])
        X=np.array(X[1:,:])
        Xm=np.mean(X,0)
        Xsd=np.std(X,0)
        Rm=np.mean(R,0)
        Rsd=np.std(R,0)
        f=np.mean(f,0)
        DATA.append([f,Rm+1j*Xm,2*np.pi*f,Rsd+1j*Xsd])
        
    return(DATA)      


def fitz0(bob,parametro,val,data):
    par=['r1', 'r2', 'dh','N'].index(parametro)
    bob=np.array(bob)
    xv=np.linspace(val[0],val[1],100)
    z0v=list()
    for x in xv:
        bobf=np.append(bob[0:par],x)
        bobf=np.append(bobf,bob[par+1:])
        z0v.append((data-theo.l0(bobf,3000))**2)
    z0v = [item for sublist in z0v for item in sublist]
    data=[]
    trace0 = go.Scatter(
    x = xv,
    y = np.real(z0v),
    mode = 'markers')
    data.append(trace0)
    fig = go.Figure(data=data)
    plotly.offline.iplot(fig)
    bobf=np.append(bob[0:par],xv[np.argmin(z0v)])
    bobf=np.append(bobf,bob[par+1:])
    return([xv[np.argmin(z0v)],bobf,theo.l0(bobf,3000).real])    




################ nuevo


def dzcorrnorm(dataraw,Vzu,Z0):
    datams=datamean(dataraw)
    datams=[datams,dataraw[1]]
    data=np.array(datams[0])
    datas=datams[1]
    
    if Vzu=='all':
        #[0] primer archivo 0[0] f 0[1]z   0[2]w 
        za=data[0][1]
        dataplot=[]
        datacorr=[]
        for i,x in enumerate(data):
            f=x[0].real
            zu=x[1]
            w=x[2]
            x0=w*Z0.imag
            z0=Z0.real+1j*x0
            dzucorr=((1/(1/zu - 1/za + 1/z0))-z0  )
            trace0 = go.Scatter(
            x =f ,
            y = (np.imag(dzucorr)/x0).real,
            mode = 'lines',
            name=datas[i],
            legendgroup=datas[i])
            dataplot.append(trace0)
            trace1 = go.Scatter(
            x = f,
            y = (np.imag(zu-za)/x0).real,
            mode = 'markers', showlegend=False,
            legendgroup=datas[i])
            #dataplot.append(trace1)
            datacorr.append(dzucorr/x0)    
          
        layout = go.Layout(
        xaxis=dict(        type='log',zeroline=False , autorange=True   ),
        yaxis=dict(        type='linear',zeroline=False,autorange=True  ))
        fig = go.Figure(data=dataplot[1:], layout=layout)
        plotly.offline.iplot(fig)

    else:
        
        za=data[0][1]
        data=data[Vzu]
        dataplot=[]
        datacorr=[]
        for i,x in enumerate(data):
            f=x[0].real
            zu=x[1]
            w=x[2]
            x0=w*Z0.imag
            z0=Z0.real+1j*x0
            dzucorr=((1/(1/zu - 1/za + 1/z0))-z0  )
            trace0 = go.Scatter(
            x = f,
            y = (np.imag(dzucorr)/x0).real,
            mode = 'lines',
            name=datas[i+1],
            legendgroup=datas[i+1])
            dataplot.append(trace0)
            trace1 = go.Scatter(
            x = f,
            y = (np.imag(zu-za)/x0).real,
            mode = 'markers',
            showlegend=False,
            legendgroup=datas[i+1])
            dataplot.append(trace1)
            datacorr.append([f,Z0.imag,dzucorr/x0])

        layout = go.Layout(
                xaxis=dict(        type='log',zeroline=False , autorange=True   ),
                yaxis=dict(        type='linear',zeroline=False,autorange=True  ))
        fig = go.Figure(data=dataplot, layout=layout)
        plotly.offline.iplot(fig)
    
    return(fig,datacorr[1:])



def z0mean(data,v): 
    """ le saca la primera iteracion """
    data2=np.array(data)
    x=data2[0]
    ni=int(x[1][-1])
    nf=int(int(x[0][-1])/ni)
    X=np.reshape(x[4]/x[5],(ni,nf))
    Y=np.reshape(x[3],(ni,nf))
    Y=np.array(Y[1:,:])
    X=np.array(X[1:,:])
    xm=np.mean(np.mean(X[:,v[0]:v[1]],0))
    ym=np.mean(np.mean(Y[:,v[0]:v[1]],0))
    z0m=(ym+1j*xm)
    return(z0m)

def l0fit(bob,parametro,val,data):
    par=['r1', 'r2', 'dh','N'].index(parametro)
    bob=np.array(bob)
    xv=np.linspace(val[0],val[1],200)
    z0v=list()
    for x in xv:
        bobf=np.append(bob[0:par],x)
        bobf=np.append(bobf,bob[par+1:])
        z0v.append((data-theo.l0(bobf,3000))**2)
    
    data=[]
    trace0 = go.Scatter(
    x = xv,
    y = np.real(z0v),
    mode = 'markers')
    data.append(trace0)
    fig = go.Figure(data=data)
    plotly.offline.iplot(fig)
    bobf=np.append(bob[0:par],xv[np.argmin(z0v)])
    bobf=np.append(bobf,bob[par+1:])
    return([xv[np.argmin(z0v)],bobf,theo.l0(bobf,3000).real])    

def znorm(datams,Vzu):
    data=np.array(datams[0])
    datas=datams[1]
    
    if Vzu=='all':
        
        za=data[0][1]
        dataplot=[]
        datacorr=[]
        for i,x in enumerate(data):
            f=x[0].real
            zu=x[1]
            w=x[2]
            x0=w*Z0.imag
            z0=Z0.real+1j*x0
            dzucorr=((1/(1/zu - 1/za + 1/z0))-z0  )
            trace0 = go.Scatter(
            x =f ,
            y = (np.imag(dzucorr)/x0).real,
            mode = 'lines',
            name=datas[i],
            legendgroup=datas[i])
            dataplot.append(trace0)
            trace1 = go.Scatter(
            x = f,
            y = (np.imag(zu-za)/x0).real,
            mode = 'markers', showlegend=False,
            legendgroup=datas[i])
            dataplot.append(trace1)
            datacorr.append([f,Z0.imag,dzucorr/x0])    
          
        layout = go.Layout(
        xaxis=dict(        type='log',zeroline=False , autorange=True   ),
        yaxis=dict(        type='linear',zeroline=False,autorange=True  ))
        fig = go.Figure(data=dataplot[2:], layout=layout)
        plotly.offline.iplot(fig)

    else:
        
        za=data[0][1]
        data=data[Vzu]
        dataplot=[]
        datacorr=[]
        for i,x in enumerate(data):
            f=x[0].real
            zu=x[1]
            w=x[2]
            x0=w*Z0.imag
            z0=Z0.real+1j*x0
            dzucorr=((1/(1/zu - 1/za + 1/z0))-z0  )
            trace0 = go.Scatter(
            x = f,
            y = (np.imag(dzucorr)/x0).real,
            mode = 'lines',
            name=datas[i+1],
            legendgroup=datas[i+1])
            dataplot.append(trace0)
            trace1 = go.Scatter(
            x = f,
            y = (np.imag(zu-za)/x0).real,
            mode = 'markers',
            showlegend=False,
            legendgroup=datas[i+1])
            dataplot.append(trace1)
            datacorr.append([f,Z0.imag,dzucorr/x0])

        layout = go.Layout(
                xaxis=dict(        type='log',zeroline=False , autorange=True   ),
                yaxis=dict(        type='linear',zeroline=False,autorange=True  ))
        fig = go.Figure(data=dataplot, layout=layout)
        plotly.offline.iplot(fig)
    
    return(fig,datacorr[1:])

def iamp(data,vrms):
    """ calculo """
    z=np.abs(data[3]+1j*data[4])
    v=vrms*np.sqrt(2)
    iamp=v/z
    return(iamp*1000)





#### nuevo
def load(path=0):
    """ carga archivos en la carpeta actual, todos deben pertenecer a un mismo experimento, mismas frecuencias y misma cantidad de repeticiones, se le puede asginar la direccion en disco de la carpeta a la variable path (tener cuidado con los //), si path=0 abre una ventana de windows para elegirla manualmente
    --------------------------------------------------------------------------------------
    devuelve una lista: 
        data[0] lista de los datos de cada archivo, cada indice es una matriz con los datos crudos de cada archivo
        
        data[1] lista con los nombres de los archivos
        
        
        
    """
    if path==0:
        root = tk.Tk()
        root.lift()
        root.attributes('-topmost',True)
        root.after_idle(root.attributes,'-topmost',False)
        folder_path = filedialog.askdirectory()
        files=list()
        print('path=   '+folder_path)
        for (dirpath, dirnames, filenames) in os.walk(folder_path):

            for i,j in enumerate(filenames):
                files.extend([dirpath + '\\'+j])
                print(i,j)
            break

        root.withdraw()
        files=[files,filenames]
        data=list()
        for file in files[0]:
            data.append(read(file,';'))
            
    else:
        folder_path = path
        files=list()
        print(folder_path)
        for (dirpath, dirnames, filenames) in os.walk(folder_path):

            for i,j in enumerate(filenames):
                files.extend([dirpath + '\\'+j])
                print(i,j)
            break
        files=[files,filenames]
        data=list()
        for file in files[0]:
            data.append(read(file,';'))
     
        
    return(list([data,files[1]]))



def getf(data):
    """ obtiene el vector de frecuencias de los datos"""
    return(data[0][0][2][:int(data[0][0][0][-1]/data[0][0][1][-1])])
	

def corr(f,bo,dataraw,Vzu='all',pltr=0):
    """ corrige y normaliza los datos, toma como input el vector de frecuencias, la info de la bobina y los datos
        devuelve una lista de arrays, cada array es la impedancia compleja corregida y normalizada para cada frecuencia, parte real y parte imaginaria
        para recuperar la parte real  (.real) e imaginaria (.imag)
    """
    datams=stats(dataraw)
    w=np.pi*2*f
    Z0=bo[-2]
    x0=w*Z0.imag
    
    if pltr==1:
        datas=datams[1]
        if Vzu=='all':
             
            za=data[0][1]
            dataplot=[]
            datacorr=[]
            for i,x in enumerate(data):
                f=x[0].real
                zu=x[1]
                w=x[2]
                x0=w*Z0.imag
                z0=Z0.real+1j*x0
                dzucorr=((1/(1/zu - 1/za + 1/z0))-z0  )
                trace0 = go.Scatter(
                x =f ,
                y = (np.imag(dzucorr)/x0).real,
                mode = 'lines',
                name=datas[i],
                legendgroup=datas[i])
                dataplot.append(trace0)
                trace1 = go.Scatter(
                x = f,
                y = (np.imag(zu-za)/x0).real,
                mode = 'markers', showlegend=False,
                legendgroup=datas[i])
                #dataplot.append(trace1)
                datacorr.append(dzucorr/x0)    

            layout = go.Layout(
            xaxis=dict(        type='log',zeroline=False , autorange=True   ),
            yaxis=dict(        type='linear',zeroline=False,autorange=True  ))
            fig = go.Figure(data=dataplot[1:], layout=layout)
            plotly.offline.iplot(fig)

        else:	
            za=data[0][1]
            data=data[Vzu]
            dataplot=[]
            datacorr=[]
            for i,x in enumerate(data):
                f=x[0].real
                zu=x[1]
                w=x[2]
                x0=w*Z0.imag
                z0=Z0.real+1j*x0
                dzucorr=((1/(1/zu - 1/za + 1/z0))-z0  )
                trace0 = go.Scatter(
                x = f,
                y = (np.imag(dzucorr)/x0).real,
                mode = 'lines',
                name=datas[i+1],
                legendgroup=datas[i+1])
                dataplot.append(trace0)
                trace1 = go.Scatter(
                x = f,
                y = (np.imag(zu-za)/x0).real,
                mode = 'markers',
                showlegend=False,
                legendgroup=datas[i+1])
                dataplot.append(trace1)
                datacorr.append([f,Z0.imag,dzucorr/x0])

            layout = go.Layout(
                    xaxis=dict(        type='log',zeroline=False , autorange=True   ),
                    yaxis=dict(        type='linear',zeroline=False,autorange=True  ))
            fig = go.Figure(data=dataplot, layout=layout)
            plotly.offline.iplot(fig)
        ret=[fig,datacorr[1:]]
        
    if pltr==0:
        x0=f*2*np.pi*bo[-1]
        z0=np.real(bo[-2])+1j*x0
        if Vzu=='all':
            #[0] primer archivo 0[0] f 0[1]z   0[2]w 
            za=datams[0][0]
            dataplot=[]
            datacorr=[]
            data=datams[1:]
            for i,x in enumerate(data):
                zu=x[0]
				   
				  
						
							
                dzucorr=((1/(1/zu - 1/za + 1/z0))-z0  )
								
				  
										   
						   
							
								   
								   
								
				  
										 
							 
							 
								   
								   
                datacorr.append(dzucorr/x0)    

						   
																				  
																				  
													 
								 
	
							

        else:

            za=data[0][0]
            data=data[Vzu]
            dataplot=[]
            datacorr=[]
            for i,x in enumerate(data):
                zu=x[1]
                dzucorr=((1/(1/zu - 1/za + 1/z0))-z0  )
                datacorr.append(dzucorr/x0)

                
        ret=list([np.array(datacorr),dataraw[1][1:]])

    return(ret)


def stats(data):
    dataz3=data[0]
    DATA=[]
    for n,x in enumerate(dataz3):
        ni=int(x[1][-1])
        nf=int(int(x[0][-1])/ni)
        X=np.reshape(x[4],(ni,nf))
        R=np.reshape(x[3],(ni,nf))
        
        R=np.array(R[1:,:])
        X=np.array(X[1:,:])
        Xm=np.mean(X,0)
        Xsd=np.std(X,0)
        Rm=np.mean(R,0)
        Rsd=np.std(R,0)
        
        DATA.append([Rm+1j*Xm,Rsd+1j*Xsd])
        
    return(DATA)      




###################################################################
###################################################################
###################################################################
###################################################################
###################################################################

import numpy as numpy
import scipy as scipy

def sig(k,sigma,f,mur):
    """ funcion auxiliar"""
    mu0 = 4*numpy.pi*1e-7;
    k2=1j*2*numpy.pi*f*mu0*mur*sigma;
    return (k*mur-numpy.sqrt(k**2+k2))/(k*mur+numpy.sqrt(k**2+k2))

def sig2(k,sigma,f,d,mur):
    """ funcion auxiliar"""
    mu0 = 4*numpy.pi*1e-7;
    k2=1j*2*numpy.pi*f*mu0*mur*sigma;
    la=numpy.sqrt(k**2+k2);
    return (((la+k*mur)*(k*mur-la)+numpy.exp(-2*la*d)*(la-k*mur)*(k*mur+la))/((la+k*mur)*(k*mur+la)+numpy.exp(-2*la*d)*(la-k*mur)*(k*mur-la)));



def sigj(k,sigma,f,mur,z):
    """ funcion auxiliar"""
    mu0 = 4*numpy.pi*1e-7;
    k2=1j*2*numpy.pi*f*mu0*mur*sigma;
    la=numpy.sqrt(k**2+k2);
    return (2*k*mur*numpy.exp(la*z)/(la+k*mur));



def sig3(k,sigma1,sigma2,f,d,mur1,mur2):
    """ funcion auxiliar"""
    mu0 = 4*numpy.pi*1e-7
    k21=1j*2*numpy.pi*f*mu0*mur1*sigma1
    k22=1j*2*numpy.pi*f*mu0*mur2*sigma2

    la1=numpy.sqrt(k**2+k21)
    la2=numpy.sqrt(k**2+k22)

    up=(la1*mur2 + la2*mur1)*(k*mur1-la1)+numpy.exp(-2*la1*d)*(la1*mur2-la2*mur1)*(k*mur1+la1)
    down=(la1*mur2 + la2*mur1)*(k*mur1+la1)+numpy.exp(-2*la1*d)*(la1*mur2-la2*mur1)*(k*mur1-la1)
    return (up/down)


def ji(k,r1,r2):
    """ funcion auxiliar"""
    return     scipy.integrate.quad(lambda x: x*scipy.special.jv(1,x) ,k*r1,k*r2)[0]

def expz(k,z1,z2):
    """ funcion auxiliar"""
    return ((numpy.exp(-k*z1)-numpy.exp(-k*z2)))/(k**(3))


# integral imaginaria

def cquad(func, a, b, **kwargs):
    """ funcion auxiliar"""
    def real_func(x):
        return numpy.real(func(x))
    def imag_func(x):
        return numpy.imag(func(x))
    real_integral = scipy.integrate.quad(real_func, a, b, **kwargs)
    imag_integral = scipy.integrate.quad(imag_func, a, b, **kwargs)
    return (real_integral[0] + 1j*imag_integral[0])

# funciones finales

def edyquad(f):
    """ funcion auxiliar"""
    aint=(1j*mu0*numpy.pi*N**2)/(L0*((r2-r1)*dh)**2)
    return aint*cquad(lambda k: sig(k,sigma,f)*(ji(k,r1,r2)*expz(k,z1,z1+dh))**2,0,5000)


def sig3(k,sigma1,sigma2,f,d,mur1,mur2):
    """ funcion auxiliar"""
    mu0 = 4*numpy.pi*1e-7
    k2_1=1j*2*numpy.pi*f*mu0*mur1*sigma1
    k2_2=1j*2*numpy.pi*f*mu0*mur2*sigma2

    la1=numpy.sqrt(k**2+k2_1)
    la2=numpy.sqrt(k**2+k2_2)

    up=(la1*mur2 + la2*mur1)*(k*mur1-la1)+numpy.exp(-2*la1*d)*(la1*mur2-la2*mur1)*(k*mur1+la1)
    down=(la1*mur2 + la2*mur1)*(k*mur1+la1)+numpy.exp(-2*la1*d)*(la1*mur2-la2*mur1)*(k*mur1-la1)
    return (up/down)
